function p = ChebyshevToLaurent(S)
% p = ChebyshevToLaurent(S)
%
% Computes the symmetric Laurent polynomial P(z) such that
%
% P(z) = S((z+z^-1)/2)
%
% where S(Z) is a regular polynomial.
%
% Method: invert the change-of-basis matrix
% to go from the canonical basis (z^n + z^-n) / 2
% to the basis ((z + z^-1) / 2)^n, for n in [0, N].
%
% The construction of the matrix is based on
% two key-properties of the Chebyshev polynomials:
% Tn((X + 1/X) / 2) = (X^n + X^-n) / 2
% Tn(X) = 2 * X * Tn-1(X) - Tn-2(X)
%
% S: Row-vector containing the coefficients of the polynomial S(Z).
%
% p: Row-vector with the coefficients of the Laurent polynomial P(z).
%
% (c) Cedric Vonesch, Biomedical Imaging Group, EPFL, 2006.07.24-2010.12.10

% The degree of S(Z)
N = numel(S) - 1;

% Construction of the change-of-basis matrix T
T = zeros(N+1, N+1);
T(N+1, N+1) = 1; % Initialize row N+1 (corresponding to T0(X))
T(N, N) = 1; % Initialize row N (corresponding to T1(X))
for n = N-1:-1:1
	T(n, :) = 2*[T(n+1, 2:end), 0] - T(n+2,:);
end

% Inversion of T
p = S / T;
p(1:end-1) = p(1:end-1) / 2; % For n > 0, we are actually working in the basis (z^n + z^-n), not (z^n + z^-n)/2
p = [p, p(end-1:-1:1)]; % Symmetric part of the Laurent polynomial