function [h, htld, g, gtld] = Gen97WavF(omega0, pad)
% [h, htld, g, gtld] = Gen97WavF(omega0, pad)
%
% Computes the scaling and wavelet filters associated with
% generalized 9/7 wavelets of parameter omega0.
%
% omega0: Frequency-parameter defining the modulated
%         polynomials to be reproduced by the scaling functions.
% pad:    If set to 'true': pad the filters with zeros if
%         necessary, so that they have the same length and
%         are exactly biorthogonal. This is useful when using
%         the Matlab functions 'dwt' and 'idwt' (default is true).
%
% h:      Synthesis scaling filter.
% htld:   Analysis scaling filter (biorthogonal to h).
% g:      Synthesis wavelet filter.
% gtld:   Analysis wavelet filter.
%
% Note: h and htld are normalized such that sum(h) = sum(htld).
%
% (c) Cedric Vonesch, Biomedical Imaging Group, EPFL, 2006.07.25-2010.11.29

if nargin < 2
	pad = true;
end

cos0 = cos(omega0);
if abs(cos0) < 1e-10
	error('Bezout equation cannot be solved when cos(omega0) = 0.');
end

alpha = [1i*omega0, 1i*omega0, -1i*omega0, -1i*omega0];

% Compute the exponential B-spline filters of parameter alpha and alphatld
r = ExpBSplineF(alpha);
rtld = r;

% Compute the biorthogonalization factors
D0 = [-5, 20, -29, 16]/256; % Explicit solution in the stationary case
z = roots(D0);
z0 = z(imag(z) == 0);
z1 = z(imag(z) > 0);

ctld = real(prod(exp(-alpha)));
q = ChebyshevToLaurent([1/cos0, -z0]/cos0);
qtld = 2 * ctld * ChebyshevToLaurent(-5/256*[1/cos0^2, -2*real(z1)/cos0, abs(z1)^2]/cos0^3);

% Compute the complete filters
h = conv(r, q);
htld = conv(rtld, qtld);

H1 = real(polyval(h, exp(1i*omega0)) * exp(-1i*(length(h)-1)/2*omega0));
Htld1 = real(polyval(htld, exp(1i*omega0)) * exp(-1i*(length(htld)-1)/2*omega0));
alpha = sign(H1) * sqrt(abs(H1/Htld1));
h = h / alpha;
htld = htld * alpha;

% Zero-pad them so that they have the same length
% and shift them so that they are exactly biorthogonal
if pad
	htld = [htld, 0];
	h = [0, h, 0, 0];
end

g = fliplr(htld) .* (-1).^(0:length(htld)-1);
gtld = fliplr(h) .* (-1).^(0:length(h)-1);