function [h, htld, g, gtld] = GenCDFWavF(alpha, alphatld, pad)
% [h, htld, g, gtld] = GenCDFWavF(alpha, alphatld, pad)
%
% Computes the scaling and wavelet filters associated with
% generalized Cohen-Daubechies-Feauveau wavelets of parameter
% alpha and alphatld. Note that, to obtain real filters, if
% alpha contains a given element, then it must also contain
% its complex conjugate, with the same multiplicity; the same
% must hold for alphatld. Also, the concatenated vector
% [alpha, -alphatld] must satisfy the symmetry conditions of
% the 'ShortestBezoutSol' routine.
%
% alpha,
% alphatld: Row-vectors defining the parameters of the
%           exponential polynomials to be reproduced by the
%           scaling functions.
% pad:      If set to 'true': pad the filters with zeros if
%           necessary, so that they have the same length and
%           are exactly biorthogonal. This is useful when using
%           the Matlab functions 'dwt' and 'idwt' (default is
%           true).
%
% h:        Synthesis scaling filter (exponential B-spline
%           filter of parameter alpha).
% htld:     Analysis scaling filter (biorthogonal to h).
% g:        Synthesis wavelet filter.
% gtld:     Analysis wavelet filter.
%
% (c) Cedric Vonesch, Biomedical Imaging Group, EPFL, 2006.07.24-2010.11.29

if nargin < 3
	pad = true;
end

N = length(alpha);
Ntld = length(alphatld);

% Compute the exponential B-spline filters of parameter alpha and alphatld
% This will also make sure that alpha and alphatld correspond to real filters
r = ExpBSplineF(alpha);
rtld = ExpBSplineF(alphatld);

% Compute the biorthogonalization factors
q = 2^(1-N);
gamma = [alpha, -alphatld];
D0 = ShortestBezoutSol(gamma);
ctld = real(prod(exp(-alphatld)));
qtld = 2^N * ctld * ChebyshevToLaurent(D0);

% Compute the complete filters
h = conv(r, q);
htld = conv(rtld, qtld);

% Zero-pad them so that they have the same length
% and shift them so that they are exactly biorthogonal
if pad
	if mod(N, 2)
		h = [zeros(1, Ntld-1), h, zeros(1, Ntld-1)];
	else
		htld = [htld, 0];
		h = [zeros(1, Ntld-1), h, zeros(1, Ntld)];
	end
end

g = fliplr(htld) .* (-1).^(0:length(htld)-1);
gtld = fliplr(h) .* (-1).^(0:length(h)-1);