function [h, g] = GenDbWavF(alpha)
% [h, g] = GenDbWavF(alpha)
%
% Computes the scaling and wavelet filters associated with
% generalized orthonormal Daubechies wavelets of parameter
% alpha. Note that, to obtain real filters, if alpha contains
% a given element, then it must also contain its complex
% conjugate, with the same multiplicity.
%
% alpha: Row-vector defining the parameters of the exponential
%        polynomials to be reproduced by the scaling functions.
%
% h:     Scaling filter (normalized s. t. sum(h) = 2).
% g:     Wavelet filter.
%
% (c) Cedric Vonesch, Biomedical Imaging Group, EPFL, 2006.07.19-2010.11.29

% Compute the exponential B-spline filter of parameter alpha
% Note that this will check whether alpha corresponds to real filters
r = ExpBSplineF(alpha);

% Compute the orthonormalizing factor
gamma = [alpha, -alpha];
D0 = ShortestBezoutSol(gamma);

% Verify that D0 is positive over [-1, 1]
z = roots(D0);
z = z(imag(z) == 0);
if any(abs(z) < 1)
	error('The shortest Bezout solution for this choice of alpha is not positive.');
end

% Symmetric spectral factorization
q = sqrt(2) * PolySqrt(D0);

% Output the filters
h = conv(r, q);
g = fliplr(h) .* (-1).^(0:length(h)-1);