function P = PolyFromRoots(z, M)
% P = PolyFromRoots(z, M)
%
% Computes the coefficients of a polynomial from the
% knowledge of its roots and their multiplicities.
%
% This routine is needed for the generation of the basis
% polynomials involved in the resolution of the Bezout
% equation in the ShortestBezoutSol routine (a special
% case of Hermite interpolation).
%
% Compared to the Matlab function 'poly', this routine
% explicitely takes into account the fact that we are
% dealing with real polynomials. Also, the multiplicities
% are handled using the separate argument M.
%
% z: Vector containing the roots of the basis polynomial
%    (note that complex (non-real) values will automatically
%    generate pairs of conjugate roots).
% M: Vector with the corresponding multiplicities.
%
% P: Basis polynomial in Matlab format, with M-th order
%    roots at the locations specified by z, and dominant
%    coefficient equal to 1.
%
% (c) Cedric Vonesch, Biomedical Imaging Group, EPFL, 2006.07.21-2010.12.10

P = 1;
for k = 1:length(z)
	if imag(z(k)) == 0
		Q = [1, -z(k)];
	else % We have a complex (non-real) root -> add its conjugate
		Q = [1, -2*real(z(k)), abs(z(k))^2];
	end
	for m = 1:M(k)
		P = conv(P, Q);
	end
end