function q = PolySqrt(S, nitermax, epsilon)
% q = PolySqrt(S, nitermax, epsilon)
%
% Performs a symmetric spectral factorization of the form:
%
% S((z+z^-1)/2) = Q(z) Q(z^-1).
%
% This algorithm was suggested by Thierry Blu. It returns
% the minimum-phase "square root" of S(Z).
%
% NOTE: this spectral factorization is possible only if
% S(Z) is positive over [-1, 1].
%
% S:        Coefficients of the polynomial S(Z).
% nitermax: Maximum number of iterations (optional).
% epsilon:  Maximum error between the coefficients of Q(z) Q(z^-1)
%           and those of S((z+z^-1)/2) (optional).
%
% q:        Coefficients of the "square root" of S(Z).
%
% (c) Cedric Vonesch, Biomedical Imaging Group, EPFL, 2006.07.21-2010.12.10

% Set default values of nitermax and epsilon
if nargin < 2
	nitermax = 100000;
end
if nargin < 3
	epsilon = 1e-10;
end

% Check that S(Z) does not take negative values over [-1, 1]
if S(end) < 0
	error(['S takes negative values over [-1, 1] (S(0) = ', num2str(S(end)), ').']);
end
r = roots(S);
while numel(r) > 0
	r_k = r(1);
	loc = (r == r_k);
	if (imag(r_k) == 0) && (abs(r_k) < 1) && (mod(sum(loc), 2) ~= 0)
		error(['S takes negative values over [-1, 1] (root ', num2str(r_k), ' has odd multiplicity).']);
	else
		r = r(not(loc));
	end
end

% Compute the Laurent polynomial P(z) = S((z+z^-1)/2)
p = ChebyshevToLaurent(S);

% Generate a first estimate of q
method = 'a';
if method == 'r' % Use simple root conversion
	r = roots(p);
	q = 1;
	for k = 1:numel(r)
		if abs(r(k)) < 1 % (Minimum phase filter)
			q = conv(q, [1, -r(k)]);
		end
	end
	q = real(q);
	q = q / sum(q) * sqrt(sum(p));
elseif method == 'a' % Use an arbitrary Laurent polynomial with half the degree of S((z+z^-1)/2)
	% NOTE: the roots of this initial polynomial should probably be inside the unit circle,
	%       in order to guarantee convergence to the minimum-phase "square root"
	q = ones(1, (numel(p)-1)/2+1);
% 	q = 2.^(0:(numel(p)-1)/2);
end

% Iterative algorithm for the determination of q
for n = 1:nitermax
	q_old = q;
	q = (q_old + deconv(p, fliplr(q_old))) / 2;
	pcur = conv(q, fliplr(q));
	if max(abs(p-pcur)) <= epsilon
		break;
	end
end
if n == nitermax
	warning('Maximum number of iterations reached in spectral factorization.');
end
if abs(q(1)/q(end)) < 1 % Flip q if we converged to the maximum-phase filter
	q = fliplr(q);
end