function [D0, C] = ShortestBezoutSol(gamma)
% [D0, C] = ShortestBezoutSol(gamma)
%
% Computes the lowest-degree solution of the Bezout
% equation C(Z) D(Z) + C(-Z) D(-Z) = 2, where C(Z)
% corresponds to the z-transform of a symmetric
% exponential B-spline scaling filter of parameter
% gamma, expressed in the Chebyshev basis.
%
% Note that, for the exponential B-spline scaling
% filter to be real-valued and symmetric, gamma
% must satisfy the following conditions:
% - If 0 is an element of gamma, then it must have
%   even multiplicity.
% - If gamma_m is an element of gamma, then so must
%   -gamma_m, conj(gamma_m) and -conj(gamma_m), with
%   the same multiplicity.
%
% Also, for the Bezout equation to be solvable,
% gamma cannot contain elements gamma_m and gamma_k
% such that gamma_m - gamma_k = i * (2n + 1) * pi.
%
% gamma: Row-vector defining C(Z).
%
% D0:    The unique solution to the Bezout equation
%        satisfying deg D0(Z) < deg C(Z).
% C:     The polynomial C(Z), for validation purposes.
%
% (c) Cedric Vonesch, Biomedical Imaging Group, EPFL, 2006.07.19-2010.11.29

% Check that gamma satisfies the symmetry constraints
% Compute the roots of C(Z) and their multiplicities z, M
% For conjugate pairs of complex (non-real) roots, only one of them appears in z
c0 = 1; % Dominant coefficient of C(Z)
k = 0; % Root counter
while numel(gamma) > 0
	gamma_m = gamma(1);
	loc = (gamma == gamma_m);
	L_m = sum(loc);
	k = k+1;
	if gamma_m == 0
		if mod(L_m, 2)
			error('gamma does not correspond to a symmetric filter (0 should have even multiplicity).');
		else
			z(k) = -1;
			M(k) = L_m/2;
			gamma = gamma(not(loc));
			c0 = 2^M(k) * c0;
		end
	elseif imag(gamma_m) == 0
		locneg = (gamma == -gamma_m);
		if sum(locneg) ~= L_m
			error(['gamma does not correspond to a symmetric filter (', num2str(gamma_m), ' and ', num2str(-gamma_m), ' should have the same multiplicity).']);
		else
			z(k) = -cosh(gamma_m);
			M(k) = L_m;
			gamma = gamma(not(loc | locneg));
			c0 = 2^M(k) * c0;
		end
	elseif real(gamma_m) == 0
		locstr = (gamma == conj(gamma_m));
		if sum(locstr) ~= L_m
			error(['gamma does not correspond to a real filter (', num2str(gamma_m), ' and ', num2str(conj(gamma_m)), ' should have the same multiplicity).']);
		else
			z(k) = -cos(imag(gamma_m));
			M(k) = L_m;
			gamma = gamma(not(loc | locstr));
			c0 = 2^M(k) * c0;
		end
	else
		locneg = (gamma == -gamma_m);
		locstr = (gamma == conj(gamma_m));
		locnegstr = (gamma == -conj(gamma_m));
		if sum(locneg) ~= L_m | sum(locstr) ~= L_m | sum(locnegstr) ~= L_m
			error(['gamma does not correspond to a real and symmetric filter (', num2str(gamma_m), ' ', num2str(-gamma_m),  ' ', num2str(conj(gamma_m)), ' ', num2str(-conj(gamma_m)), ' should have the same multiplicity).']);
		else
			z(k) = -cosh(gamma_m);
			M(k) = L_m;
			c0 = 2^(2*M(k)) * c0; % Double multiplicity due to complex-conjugate roots
			gamma = gamma(not(loc | locneg | locstr | locnegstr));
		end
	end
end
C = c0 * PolyFromRoots(z, M);

method = 'm';
if method == 'f' % Partial fraction decomposition
	D0 = 0;
	for k = 1:numel(z) % For each pole z_k
		H = zeros([M(k), M(k)]);
		rts = not(z == z(k));
		P = c0 * conv(PolyFromRoots(z(rts), M(rts)), (-1).^(numel(C)-1:-1:0).*C); % Compute the polynomial P_{k, M_k}
		if imag(z(k))
			Q = [1, -conj(z(k))];
			for m = 1:M(k)
				P = conv(P, Q);
			end		
		end
		for ell = 0:M(k)-1 % Compute its derivatives and evaluate them at z_k
			if ell > 0
				P = polyder(P);
			end
			H(ell+1, M(k)) = polyval(P, z(k));
			for d = 1:M(k)-ell-1 % Fill the corresponding diagonal of the "Hermite matrix"
				H(ell+1+d, M(k)-d) = (ell+d) * H(ell+d, M(k)-d+1);
			end
		end
		a = H \ [2; zeros(M(k)-1, 1)]; % Invert the matrix to get the coefficients a_{k, m}
		% NOTE: we could easily compute these coefficients manually (the matrix is triangular)

		if imag(z(k)) == 0
			P = c0 * PolyFromRoots(z(rts), M(rts));
			P = (-1).^(numel(P)-1:-1:0) .* P;
			Q = [-1, -z(k)];
			D0 = PolyAdd(D0, a(M(k)) * P);
			for m = M(k)-1:-1:1 % Add the terms corresponding to the P_{k, m}'s to the current estimate of D_0
				P = conv(P, Q);
				D0 = PolyAdd(D0, a(m) * P);
			end
		else % We have a complex (non-real) root -> take into account its conjugate
			P = c0 * PolyFromRoots(z(rts), M(rts));
			Q = [1, -conj(z(k))];
			for m = 1:M(k)
				P = conv(P, Q);
			end
			P = (-1).^(numel(P)-1:-1:0) .* P;
			Q = [-1, -z(k)];
			D0 = PolyAdd(D0, 2*real(a(M(k)) * P));
			for m = M(k)-1:-1:1 % Add the terms corresponding to the P_{k, m}'s to the current estimate of D_0
				P = conv(P, Q);
				D0 = PolyAdd(D0, 2*real(a(m) * P));
			end
		end
	end
elseif method == 'm' % Matrix inversion in the canonical basis
	Cconv = convmtx(C, numel(C)-1);
	Cconv = Cconv(:, fliplr(end:-2:1));
	D0 = [zeros(1, numel(C)-2), 1] / Cconv;
end